//+------------------------------------------------------------------+
//|                                                       MTF_MA.mq5 |
//|                                                        avoitenko |
//|                        https://login.mql5.com/en/users/avoitenko |
//+------------------------------------------------------------------+
#property copyright "avoitenko"
#property link      "https://login.mql5.com/en/users/avoitenko"
#property version   "1.00"

#property indicator_separate_window

#property indicator_buffers 3
#property indicator_plots   1
//---
#property indicator_type1  DRAW_LINE
#property indicator_color1 clrSteelBlue
#property indicator_width1 2
#property indicator_label1 "Line"

#include <MovingAverages.mqh>

//--- input parameters
input ENUM_TIMEFRAMES TimeFrame=PERIOD_H1; // Timeframe
input ushort RsiPeriod=14; // RSI Period
input ushort SmaSmoothing=5; // SMA Period

//--- buffers
double rsiBuffer[];
double maBuffer[];
double bars[];
datetime TimeBegining[];
//--- global var's
bool returnBars;
bool calculateValue;
string IndicatorFileName;

int rsi_handle;
int ret_handle;
int calc_handle;

ENUM_TIMEFRAMES iTimeFrame;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {

   iTimeFrame=TimeFrame;
   if(iTimeFrame>=0 && PeriodSeconds(iTimeFrame)<PeriodSeconds(_Period)) iTimeFrame=_Period;
//---
   SetIndexBuffer(0,maBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,rsiBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(2,bars,INDICATOR_CALCULATIONS);
//---
   ArraySetAsSeries(maBuffer,true);
   ArraySetAsSeries(rsiBuffer,true);
   ArraySetAsSeries(bars,true);
//---
   IndicatorFileName=MQL5InfoString(MQL5_PROGRAM_NAME);

   returnBars=(iTimeFrame==-1); if(returnBars) return(0);
   calculateValue=(iTimeFrame==-2);

   if(calculateValue)
     {
      rsi_handle=iRSI(NULL,_Period,RsiPeriod,PRICE_CLOSE);
      if(rsi_handle==INVALID_HANDLE){Print("Error handle #3");return(-1);}
      return(0);
     }

   if(calculateValue) return(0);

   ret_handle=iCustom(NULL,iTimeFrame,IndicatorFileName,-1,RsiPeriod,SmaSmoothing);
   if(ret_handle==INVALID_HANDLE){Print("Error  handle #1");return(-1);}

   calc_handle=iCustom(NULL,iTimeFrame,IndicatorFileName,-2,RsiPeriod,SmaSmoothing);
   if(calc_handle==INVALID_HANDLE){Print("Error handle #2");return(-1);}

   rsi_handle=iRSI(NULL,iTimeFrame,RsiPeriod,PRICE_CLOSE);
   if(rsi_handle==INVALID_HANDLE){Print("Error handle #3");return(-1);}

   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {

   ArraySetAsSeries(time,true);

   int limit;
   if(prev_calculated>rates_total || prev_calculated<=0)
     {
      for(int i=rates_total-1; i>=0; i--)
        {
         maBuffer[i]=EMPTY_VALUE;
         rsiBuffer[i]=EMPTY_VALUE;
        }
      limit=rates_total-1;
     }
   else limit=rates_total-prev_calculated;
//Print("1 ",limit);
   if(returnBars){rsiBuffer[0]=limit; return(0);}

//---    
   if(calculateValue || iTimeFrame==PERIOD_CURRENT || iTimeFrame==_Period)
     {
      if(CopyBuffer(rsi_handle,0,0,rates_total,rsiBuffer)!=rates_total) return(0);
      SimpleMAOnBuffer(rates_total,prev_calculated,RsiPeriod,SmaSmoothing,rsiBuffer,maBuffer);
      return(rates_total);
     }

//---    
   CopyBuffer(ret_handle,1,0,1,bars);

   limit=(int)MathMin(limit,bars[0]);

   CopyTime(_Symbol,iTimeFrame,(int)bars[0]-1,1,TimeBegining);
   CopyBuffer(calc_handle,0,0,(int)bars[0],bars);

   for(int i=limit; i>=0; i--)
     {
      maBuffer[i]=EMPTY_VALUE;
      if(time[i]>=TimeBegining[0])
        {
         int y=iBarShift(NULL,iTimeFrame,time[i]);
         if(y>=0) maBuffer[i]=bars[y];
        }
     }

   return(rates_total);
  }
//+------------------------------------------------------------------+
//|   iBarShift                                                      |
//+------------------------------------------------------------------+
int iBarShift(string symbol,ENUM_TIMEFRAMES timeframe,datetime time,bool exact=false)
  {
   if(time<0) return(-1);
   datetime Arr[];
   CopyTime(symbol,timeframe,0,1,Arr);
   datetime time1=Arr[0];
   if(CopyTime(symbol,timeframe,time,time1,Arr)>0)
     {
      if(ArraySize(Arr)>2) return(ArraySize(Arr)-1);
      if(time<time1) return(1);
      else return(0);
     }
   else return(-1);
  }
//+------------------------------------------------------------------+
